//-----------------------------------------------------------------------------
//  Copyright (C) 2014-2015 Thomas S. Ullrich
//
//  This file is part of "MineSweeper".
//
//  This file may be used under the terms of the GNU General Public License.
//  This project is free software; you can redistribute it and/or modify it
//  under the terms of the GNU General Public License.
//
//  Author: Thomas S. Ullrich
//  Last update: July 29, 2014
//-----------------------------------------------------------------------------
#include <QtCore/QVariant>
#include <QtWidgets/QAction>
#include <QtWidgets/QApplication>
#include <QtWidgets/QButtonGroup>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QTabWidget>
#include <QtWidgets/QTableWidget>
#include <QtWidgets/QWidget>
#include <QSettings>
#include <QMessageBox>
#include "msHighscoreList.h"
#include "msHighscoreEntry.h"
#include "msSettings.h"
#include <iostream>

using namespace std;

#define PR(x)  cout << #x << " = " << (x) << endl;

msHighscoreList::msHighscoreList(QWidget *parent) : QDialog(parent)
{
    setWindowTitle(tr("MineSweeper - High Scores"));
    setWindowModality(Qt::ApplicationModal);

    QSizePolicy sizePolicy(QSizePolicy::Preferred, QSizePolicy::Preferred);
    sizePolicy.setHorizontalStretch(0);
    sizePolicy.setVerticalStretch(0);
    setSizePolicy(sizePolicy);

    mGridLayoutDialog = new QGridLayout(this);
    mTabWidget = new QTabWidget(this);

    //
    //  Table for small size
    //
    mSmallTab = new QWidget();
    mGridLayoutSmall = new QGridLayout(mSmallTab);
    mSmallTable = new QTableWidget(mSmallTab);
    setupTable(mSmallTable);
    mGridLayoutSmall->addWidget(mSmallTable, 0, 0, 1, 1);
    mTabWidget->addTab(mSmallTab, QString());
    mTabWidget->setTabText(mTabWidget->indexOf(mSmallTab), tr("Small"));
    
    //
    //  Table for medium size
    //
    mMediumTab = new QWidget();
    mGridLayoutMedium = new QGridLayout(mMediumTab);
    mMediumTable = new QTableWidget(mMediumTab);
    setupTable(mMediumTable);
    mGridLayoutMedium->addWidget(mMediumTable, 0, 0, 1, 1);
    mTabWidget->addTab(mMediumTab, QString());
    mTabWidget->setTabText(mTabWidget->indexOf(mMediumTab), tr("Medium"));
    
    //
    //  Table for large size
    //
    mLargeTab = new QWidget();
    mGridLayoutLarge = new QGridLayout(mLargeTab);
    mLargeTable = new QTableWidget(mLargeTab);
    setupTable(mLargeTable);
    mGridLayoutLarge->addWidget(mLargeTable, 0, 0, 1, 1);
    mTabWidget->addTab(mLargeTab, QString());
    mTabWidget->setTabText(mTabWidget->indexOf(mLargeTab), tr("Large"));
    mGridLayoutDialog->addWidget(mTabWidget, 0, 0, 1, 1);
    
    //
    //  Buttons
    //
    mButtonBox = new QDialogButtonBox(this);
    mButtonBox->setOrientation(Qt::Horizontal);
    // mButtonBox->setStandardButtons(QDialogButtonBox::Reset|QDialogButtonBox::Close);
    QPushButton *closeButton = mButtonBox->addButton("Close", QDialogButtonBox::AcceptRole);
    closeButton->setDefault(true);
    (void) mButtonBox->addButton("Clear List", QDialogButtonBox::RejectRole);
    
    mGridLayoutDialog->addWidget(mButtonBox, 1, 0, 1, 1);

    connect(mButtonBox, SIGNAL(accepted()), this, SLOT(accept()));
    connect(mButtonBox, SIGNAL(rejected()), this, SLOT(erase()));
    
    mTabWidget->setCurrentIndex(0);  // will get changed to default to current used size
    resize(500, 400);
}

msHighscoreList::~msHighscoreList() { /* no op */ }

void msHighscoreList::erase()
{
    QMessageBox::StandardButton answer = QMessageBox::question(this, "MineSweeper - Confirm",
                                       "Are you sure you want to erase the high score list for the respective game size?\nThis cannot be undone.");
    if (answer == QMessageBox::Yes) {
        if (mTabWidget->currentIndex() == static_cast<int>(msSettings::msGameSize::small))
            mList_Small.clear();
        else if (mTabWidget->currentIndex() == static_cast<int>(msSettings::msGameSize::medium))
            mList_Medium.clear();
        else if (mTabWidget->currentIndex() == static_cast<int>(msSettings::msGameSize::large))
            mList_Large.clear();
    }
    sync();
    return;
}

void msHighscoreList::setupTable(QTableWidget *table)
{
    table->setSortingEnabled(false);
    table->setLineWidth(0);
    table->setMidLineWidth(0);
    table->setDragDropOverwriteMode(false);
    table->setSelectionMode(QAbstractItemView::NoSelection);
    table->setTextElideMode(Qt::ElideLeft);
    table->setShowGrid(false);
    table->verticalHeader()->setSectionResizeMode(QHeaderView::Fixed);
    table->verticalHeader()->setDefaultSectionSize(22);
    table->verticalHeader()->hide();
    table->setEditTriggers(QAbstractItemView::NoEditTriggers);
    table->setVerticalScrollBarPolicy(Qt::ScrollBarAsNeeded);
    table->setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOff);

    table->setColumnCount(4);
    QTableWidgetItem *rankItem = new QTableWidgetItem();
    table->setHorizontalHeaderItem(0, rankItem);
    rankItem->setText(tr("Rank"));
    
    QTableWidgetItem *dateItem = new QTableWidgetItem();
    table->setHorizontalHeaderItem(1, dateItem);
    dateItem->setText(tr("Date"));
    
    QTableWidgetItem *playerItem = new QTableWidgetItem();
    table->setHorizontalHeaderItem(2, playerItem);
    playerItem->setText(tr("Player"));
    
    QTableWidgetItem *durationItem = new QTableWidgetItem();
    table->setHorizontalHeaderItem(3, durationItem);
    durationItem->setText(tr("Time"));
    
    table->setRowCount(10);
    
    QHeaderView* hHeader = table->horizontalHeader();
    hHeader->setSectionResizeMode(0, QHeaderView::ResizeToContents);
    hHeader->setSectionResizeMode(1, QHeaderView::ResizeToContents);
    hHeader->setSectionResizeMode(2, QHeaderView::ResizeToContents);
    hHeader->setSectionResizeMode(3, QHeaderView::Stretch);
}

void msHighscoreList::load(QSettings* s)
{
    char text[8];
    mList_Small.clear();
    mList_Medium.clear();
    mList_Large.clear();
    
    s->beginGroup("/HighScores");
    
    s->beginGroup("/Small");
    unsigned int n = s->value("/NumberOfSmallScores", 0).toUInt();
    for (unsigned int i=0; i<n; i++) {
        sprintf(text,"/%d", i);
        s->beginGroup(text);
        time_t tg = s->value("/Time", 0).toUInt();
        time_t td = s->value("/Duration", 0).toUInt();
        QString name = s->value("/Player", "").toString();
        s->endGroup();
        msHighscoreEntry entry(name, tg, td, msSettings::msGameSize::small);
        mList_Small.push_back(entry);
        sort(mList_Small.begin(), mList_Small.end(), msHighscoreEntryCompare());
    }
    s->endGroup();  // Small
    
    s->beginGroup("/Medium");
    n = s->value("/NumberOfMediumScores", 0).toUInt();
    for (unsigned int i=0; i<n; i++) {
        sprintf(text,"/%d", i);
        s->beginGroup(text);
        time_t tg = s->value("/Time", 0).toUInt();
        time_t td = s->value("/Duration", 0).toUInt();
        QString name = s->value("/Player", "").toString();
        s->endGroup();
        msHighscoreEntry entry(name, tg, td, msSettings::msGameSize::medium);
        mList_Medium.push_back(entry);
        sort(mList_Medium.begin(), mList_Medium.end(), msHighscoreEntryCompare());
    }
    s->endGroup();  // Medium
    
    s->beginGroup("/Large");
    n = s->value("/NumberOfLargeScores", 0).toUInt();
    for (unsigned int i=0; i<n; i++) {
        sprintf(text,"/%d", i);
        s->beginGroup(text);
        time_t tg = s->value("/Time", 0).toUInt();
        time_t td = s->value("/Duration", 0).toUInt();
        QString name = s->value("/Player", "").toString();
        s->endGroup();
        msHighscoreEntry entry(name, tg, td, msSettings::msGameSize::large);
        mList_Large.push_back(entry);
        sort(mList_Large.begin(), mList_Large.end(), msHighscoreEntryCompare());
    }
    s->endGroup();  // Large
    
    s->endGroup();  // HighScores
}

void msHighscoreList::save(QSettings* s)
{
    char text[4];
    
    s->beginGroup("/HighScores");
    
    s->beginGroup("/Small");
    s->setValue("//NumberOfSmallScores", static_cast<unsigned int>(mList_Small.size()));
    for (unsigned int i=0; i<mList_Small.size(); i++) {
        sprintf(text,"/%d", i);
        s->beginGroup(text);
        s->setValue("/Time", static_cast<unsigned int>(mList_Small[i].timeOfGame()));
        s->setValue("/Duration",  static_cast<unsigned int>(mList_Small[i].durationOfGame()));
        s->setValue("/Player", mList_Small[i].name());
        s->endGroup();
    }
    s->endGroup();  // Small
    
    s->beginGroup("/Medium");
    s->setValue("/NumberOfMediumScores", static_cast<unsigned int>(mList_Medium.size()));
    for (unsigned int i=0; i<mList_Medium.size(); i++) {
        sprintf(text,"/%d", i);
        s->beginGroup(text);
        s->setValue("/Time", static_cast<unsigned int>(mList_Medium[i].timeOfGame()));
        s->setValue("/Duration", static_cast<unsigned int>(mList_Medium[i].durationOfGame()));
        s->setValue("/Player", mList_Medium[i].name());
        s->endGroup();
    }
    s->endGroup();   // Medium
    
    s->beginGroup("/Large");
    s->setValue("//NumberOfLargeScores", static_cast<unsigned int>(mList_Large.size()));
    for (unsigned int i=0; i<mList_Large.size(); i++) {
        sprintf(text,"/%d", i);
        s->beginGroup(text);
        s->setValue("/Time", static_cast<unsigned int>(mList_Large[i].timeOfGame()));
        s->setValue("/Duration", static_cast<unsigned int>(mList_Large[i].durationOfGame()));
        s->setValue("/Player", mList_Large[i].name());
        s->endGroup();
    }
    s->endGroup();  // Large
    
    s->endGroup();  // HighScores
}

bool msHighscoreList::insertEntry(msHighscoreEntry& entry)
{
    msHighscoreEntryCompare cmp;
    
    vector<msHighscoreEntry> *list;
    if (entry.gameSize() == msSettings::msGameSize::small)
        list = &mList_Small;
    else if (entry.gameSize() == msSettings::msGameSize::medium)
        list = &mList_Medium;
    else if (entry.gameSize() == msSettings::msGameSize::large)
        list = &mList_Large;
    else
        return false;

    if (list->size() < static_cast<unsigned int>(msSettings::mMaxHighscoreItems)) {   // highscore list not full, always add new
        list->push_back(entry);
        sort(list->begin(), list->end(), msHighscoreEntryCompare());
        return true;
    }
    else {
        if (cmp(entry, (*list)[msSettings::mMaxHighscoreItems-1])) {
            (*list)[msSettings::mMaxHighscoreItems-1] = entry;  // supercedes existing entry, replace last
            sort(list->begin(), list->end(), msHighscoreEntryCompare());
            return true;
        }
    }
    return false;
    
}

void msHighscoreList::syncAndShow()
{
    sync();
    show();
}

void msHighscoreList::sync()
{
    QString rank;
    
    for (int k=0; k<3; k++) {
        QTableWidget *table;
        vector<msHighscoreEntry>  *list;
        
        if (k == static_cast<int>(msSettings::msGameSize::small)) {
            table = mSmallTable;
            list = &mList_Small;
        }
        else if (k == static_cast<int>(msSettings::msGameSize::medium)) {
            table = mMediumTable;
            list = &mList_Medium;
        }
        else if (k == static_cast<int>(msSettings::msGameSize::large)) {
            table = mLargeTable;
            list = &mList_Large;
        }
        else
            return;
        table->clearContents();
        QTableWidgetItem *item;
        for (unsigned int i=0; i<list->size(); i++) {
            bool doHighlight = (*list)[i].isLast();
            (*list)[i].setIsLast(false);   // highlight only once
            rank.setNum(i+1);
            item = new QTableWidgetItem(rank);
            if (doHighlight) item->setForeground(Qt::red);
            table->setItem(i, 0, item);
            table->item(i, 0)->setTextAlignment(Qt::AlignHCenter);
            item = new QTableWidgetItem((*list)[i].dateOfGame());
            if (doHighlight) item->setForeground(Qt::red);
            table->setItem(i, 1, item);
            table->item(i, 1)->setTextAlignment(Qt::AlignHCenter);
            item = new QTableWidgetItem((*list)[i].name());
            if (doHighlight) item->setForeground(Qt::red);
            table->setItem(i, 2, item);
            table->item(i, 2)->setTextAlignment(Qt::AlignHCenter);
            time_t d = (*list)[i].durationOfGame();
            time_t m = d/60;
            time_t s = d%60;
            char text[8];
            sprintf(text, "%02ld:%02ld", m, s);
            item = new QTableWidgetItem(text);
            if (doHighlight) item->setForeground(Qt::red);
            table->setItem(i, 3, item);
            table->item(i, 3)->setTextAlignment(Qt::AlignHCenter);
        }
        update();
    }
}

bool msHighscoreList::isHighscore(time_t d, msSettings::msGameSize s)
{
    vector<msHighscoreEntry> *list;
    if (s == msSettings::msGameSize::small)
        list = &mList_Small;
    else if (s == msSettings::msGameSize::medium)
        list = &mList_Medium;
    else if (s == msSettings::msGameSize::large)
        list = &mList_Large;
    else
        return false;

    if (list->size() < static_cast<unsigned int>(msSettings::mMaxHighscoreItems)) return true;
    
    time_t longestDuration = 0;
    for (unsigned int i=0; i< list->size(); i++) {
        if ( (*list)[i].durationOfGame() > longestDuration)
            longestDuration = (*list)[i].durationOfGame();
    }
    return (d < longestDuration);
}

void msHighscoreList::show()
{
    mTabWidget->setCurrentIndex(static_cast<int>(msSettings::mCurrentGameSize));
    QDialog::show();
}