//-----------------------------------------------------------------------------
//  Copyright (C) 2014-2016 Thomas S. Ullrich
//
//  This file is part of "MineSweeper".
//
//  This file may be used under the terms of the GNU General Public License.
//  This project is free software; you can redistribute it and/or modify it
//  under the terms of the GNU General Public License.
//
//  Author: Thomas S. Ullrich
//  Last update: May 3, 2016
//-----------------------------------------------------------------------------
#include "msPreferencesDialog.h"
#include <QtCore/QVariant>
#include <QtWidgets/QAction>
#include <QtWidgets/QApplication>
#include <QtWidgets/QButtonGroup>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QSpinBox>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QFormLayout>
#include <QtWidgets/QGraphicsView>
#include <QtWidgets/QGraphicsScene>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QLabel>
#include <QtWidgets/QListWidget>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QTabWidget>
#include <QtWidgets/QWidget>
#include <QGraphicsRectItem>
#include <QColorDialog>
#include <QListWidgetItem>
#include <QPaintEngine>
#include <QSvgRenderer>
#include <QGraphicsSvgItem>
#include <iostream>
#include "msSettings.h"

using namespace std;

#define PR(x) cout << #x << " = " << (x) << endl;

msPreferencesDialog::msPreferencesDialog(QWidget *parent) : QDialog(parent)
{
    mSelectedCellSize = msSettings::mCellSize;

    //
    //  Create tab widget and layout
    //
    mDialogGridLayout = new QGridLayout(this);
    mTabWidget = new QTabWidget(this);
    setWindowTitle(tr("MineSweeper - Preferences"));
    setModal(true);

    //
    //  Tab: Game Options
    //
    
    mGameOptionsTab = new QWidget();
    mGameOptionsTabGridLayout = new QGridLayout(mGameOptionsTab);
    
    mEasyStartupCheckBox = new QCheckBox(mGameOptionsTab);
    mGameOptionsTabGridLayout->addWidget(mEasyStartupCheckBox, 0, 0, 1, 1);
    mEasyStartupCheckBox->setChecked(msSettings::mEasyStart);
    mEasyStartupCheckBox->setText(tr("Easy startup (reveal mine free patch at startup)"));
    
    mQuestionableFlagCheckBox = new QCheckBox(mGameOptionsTab);
    mGameOptionsTabGridLayout->addWidget(mQuestionableFlagCheckBox, 1, 0, 1, 1);
    mQuestionableFlagCheckBox->setChecked(msSettings::mAllowQuestionableFlag);
    mQuestionableFlagCheckBox->setText(tr("Allow to flag cells as questionable"));

    mAllowOopsiesCheckBox = new QCheckBox(mGameOptionsTab);
    mGameOptionsTabGridLayout->addWidget(mAllowOopsiesCheckBox, 2, 0, 1, 1);
    mAllowOopsiesCheckBox->setChecked(msSettings::mNumberOfAllowedOopsies);
    mAllowOopsiesCheckBox->setText(tr("Submit a warning before a mine cell is revealed (3 times only)"));
    
    QLabel *note = new QLabel("Note: If more than one 'cheat' was used, the game is barred from "
                              "entering the highscore list.", mGameOptionsTab);
    note->setIndent(20);
    note->setWordWrap(true);
    QFont noteFont  = note->font();
    noteFont.setPointSize(noteFont.pointSize()-1);
    note->setFont(noteFont);
    mGameOptionsTabGridLayout->addWidget(note, 3, 0, 1, 1);
    
    mVerticalSpacer1 = new QSpacerItem(20, 198, QSizePolicy::Minimum, QSizePolicy::Expanding);
    mGameOptionsTabGridLayout->addItem(mVerticalSpacer1, 4, 0, 1, 1);
    
    mTabWidget->addTab(mGameOptionsTab, QString());
    mTabWidget->setTabText(mTabWidget->indexOf(mGameOptionsTab), tr("Game Options"));

    //
    //  Tab: Cell Appearance (Colors & Size) 
    //
    
    mCellAppearanceTab = new QWidget();
    mCellAppearanceTabGridLayout = new QGridLayout(mCellAppearanceTab);
    
    mGraphicsView = new QGraphicsView(mCellAppearanceTab);
   
    mGraphicsScene = new QGraphicsScene(mGraphicsView);
    mGraphicsView->setScene(mGraphicsScene);
    mGraphicsView->setCacheMode(QGraphicsView::CacheBackground);
    mGraphicsView->setFrameStyle(QFrame::NoFrame);
    mGraphicsView->setViewportUpdateMode(QGraphicsView::FullViewportUpdate);
    mGraphicsView->setAcceptDrops(false);
    mGraphicsView->setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
    mGraphicsView->setVerticalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
    mGraphicsView->setRenderHint(QPainter::HighQualityAntialiasing, true);
    mGraphicsView->setSceneRect(0, 0, 4*msSettings::mMaxCellSize+1, 4*msSettings::mMaxCellSize+1);
    
    mSelectedBackgroundColor = msSettings::mBackgroundColor;
    mSelectedWaterColor = msSettings::mWaterColor;
    
    mCellAppearanceTabGridLayout->addWidget(mGraphicsView, 0, 0, 2, 1);
    
    mFormLayout1 = new QFormLayout();
    
    mWaterColorLabel = new QLabel(mCellAppearanceTab);
    mWaterColorLabel->setText(tr("Water Surface Color:"));
    mFormLayout1->setWidget(0, QFormLayout::LabelRole, mWaterColorLabel);
    mWaterColorComboBox = new QComboBox(mCellAppearanceTab);
    mFormLayout1->setWidget(0, QFormLayout::FieldRole, mWaterColorComboBox);
    mWaterColorComboBox->clear();
    mWaterColorComboBox->insertItems(0, QStringList()
                                     << tr("Current", 0)
                                     << tr("Default", 0)
                                     << tr("New", 0)
                                     );
    mWaterColorComboBox->setCurrentIndex(0); // current
    connect(mWaterColorComboBox, SIGNAL(activated(int)), this, SLOT(waterColorComboActivated(int)));

    mBackgroundColorLabel = new QLabel(mCellAppearanceTab);
    mBackgroundColorLabel->setText(tr("Background Color:"));
    mFormLayout1->setWidget(1, QFormLayout::LabelRole, mBackgroundColorLabel);
    mBackgroundColorComboBox = new QComboBox(mCellAppearanceTab);
    mFormLayout1->setWidget(1, QFormLayout::FieldRole, mBackgroundColorComboBox);
    mBackgroundColorComboBox->clear();
    mBackgroundColorComboBox->insertItems(0, QStringList()
                                          << tr("Current")
                                          << tr("Default")
                                          << tr("New")
                                          );
    mBackgroundColorComboBox->setCurrentIndex(0); // current
    connect(mBackgroundColorComboBox, SIGNAL(activated(int)), this, SLOT(backgroundColorComboActivated(int)));
   
    mCellSizeLabel = new QLabel(mCellAppearanceTab);
    mCellSizeLabel->setText(tr("Cell Size:"));
    mFormLayout1->setWidget(2, QFormLayout::LabelRole, mCellSizeLabel);
    mCellSizeSpinBox = new QSpinBox(mCellAppearanceTab);
    mFormLayout1->setWidget(2, QFormLayout::FieldRole, mCellSizeSpinBox);
    mCellSizeSpinBox->setRange(msSettings::mMinCellSize,msSettings::mMaxCellSize);
    mCellSizeSpinBox->setValue(mSelectedCellSize);
    connect(mCellSizeSpinBox, SIGNAL(valueChanged(int)), this, SLOT(cellSizeBoxActivated(int)));
   
    mCellAppearanceTabGridLayout->addLayout(mFormLayout1, 0, 1, 1, 1);
    mHorizontalSpacer1 = new QSpacerItem(60, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);
    mCellAppearanceTabGridLayout->addItem(mHorizontalSpacer1, 0, 2, 1, 1);
    mVerticalSpacer3 = new QSpacerItem(20, 157, QSizePolicy::Minimum, QSizePolicy::Expanding);
    mCellAppearanceTabGridLayout->addItem(mVerticalSpacer3, 1, 1, 2, 1);
    mVerticalSpacer2 = new QSpacerItem(20, 71, QSizePolicy::Minimum, QSizePolicy::Expanding);
    mCellAppearanceTabGridLayout->addItem(mVerticalSpacer2, 2, 0, 1, 1);
    
    mTabWidget->addTab(mCellAppearanceTab, QString());
    mTabWidget->setTabText(mTabWidget->indexOf(mCellAppearanceTab), tr("Cell Appearance"));

    //
    //  Tab: Pictograms for Mines and Flags
    //
    
    mPictoTab = new QWidget();
    mPictoTabGridLayout = new QGridLayout(mPictoTab);
    
    mMinesGroupBox = new QGroupBox(mPictoTab);
    mMinesGroupBox->setTitle(tr("Mines"));
    mMinesListWidget = new QListWidget(mMinesGroupBox);
    mMinesListWidget->setMovement(QListView::Static);
    mMinesListWidget->setViewMode(QListView::IconMode); // or List Mode (top to bottom)
    mMinesListWidget->setSelectionMode(QAbstractItemView::SingleSelection);
    mMinesListWidget->setUniformItemSizes(true);
    mMinesGroupBoxGridLayout = new QGridLayout(mMinesGroupBox);
    mMinesGroupBoxGridLayout->addWidget(mMinesListWidget, 0, 0, 1, 1);
    mPictoTabGridLayout->addWidget(mMinesGroupBox, 0, 0, 1, 1);
    //mMinesListWidget->setIconSize(QSize(0.8*msSettings::mCellSize, 0.8*msSettings::mCellSize));
    mMinesListWidget->setWrapping(false);
    mMinesListWidget->setMaximumHeight(75);
    mMinesListWidget->setSpacing(6);
    
    addListItem(mMinesListWidget, mMineItems, (msSettings::mCurrentMineType == msSettings::msMineType::mine1), ":/images/mine1.svg", "mine 1");
    addListItem(mMinesListWidget, mMineItems, (msSettings::mCurrentMineType == msSettings::msMineType::mine2), ":/images/mine2.svg", "mine 2");
    addListItem(mMinesListWidget, mMineItems, (msSettings::mCurrentMineType == msSettings::msMineType::mine3), ":/images/mine3.svg", "mine 3");
    addListItem(mMinesListWidget, mMineItems, (msSettings::mCurrentMineType == msSettings::msMineType::bomb1), ":/images/bomb1.svg", "bomb 1");
    addListItem(mMinesListWidget, mMineItems, (msSettings::mCurrentMineType == msSettings::msMineType::bomb2), ":/images/bomb2.svg", "bomb 2");
    addListItem(mMinesListWidget, mMineItems, (msSettings::mCurrentMineType == msSettings::msMineType::bomb3), ":/images/bomb3.svg", "bomb 3");
    
    mFlagsGroupBox = new QGroupBox(mPictoTab);
    mFlagsGroupBox->setTitle(tr("Flags"));
    mFlagsListWidget = new QListWidget(mFlagsGroupBox);
    mFlagsListWidget->setMovement(QListView::Static);
    mFlagsListWidget->setViewMode(QListView::IconMode);
    mFlagsListWidget->setSelectionMode(QAbstractItemView::SingleSelection);
    mFlagsListWidget->setUniformItemSizes(true);
    mFlagsGroupBoxGridLayout = new QGridLayout(mFlagsGroupBox);
    mFlagsGroupBoxGridLayout->addWidget(mFlagsListWidget, 0, 0, 1, 1);
    mPictoTabGridLayout->addWidget(mFlagsGroupBox, 1, 0, 1, 1);
    // mFlagsListWidget->setIconSize(QSize(0.8*msSettings::mCellSize, 0.8*msSettings::mCellSize));
    mFlagsListWidget->setWrapping(false);
    mFlagsListWidget->setMaximumHeight(75);
     mFlagsListWidget->setSpacing(6);
 
    addListItem(mFlagsListWidget, mFlagItems, (msSettings::mCurrentFlagType == msSettings::msFlagType::flag1), ":/images/flag1.svg", "flag 1");
    addListItem(mFlagsListWidget, mFlagItems, (msSettings::mCurrentFlagType == msSettings::msFlagType::flag2), ":/images/flag2.svg", "flag 2");
    addListItem(mFlagsListWidget, mFlagItems, (msSettings::mCurrentFlagType == msSettings::msFlagType::flag3), ":/images/flag3.svg", "flag 3");
    addListItem(mFlagsListWidget, mFlagItems, (msSettings::mCurrentFlagType == msSettings::msFlagType::warning1), ":/images/warning1.svg", "sign 1");
    addListItem(mFlagsListWidget, mFlagItems, (msSettings::mCurrentFlagType == msSettings::msFlagType::warning2), ":/images/warning2.svg", "sign 2");
    addListItem(mFlagsListWidget, mFlagItems, (msSettings::mCurrentFlagType == msSettings::msFlagType::warning3), ":/images/warning3.svg", "sign 3");
    
    mTabWidget->addTab(mPictoTab, QString());
    mDialogGridLayout->addWidget(mTabWidget, 0, 0, 1, 1);
    mTabWidget->setTabText(mTabWidget->indexOf(mPictoTab), tr("Pictograms"));
    
    //
    //  Buttons
    //
    mButtonBox = new QDialogButtonBox(this);
    mButtonBox->setOrientation(Qt::Horizontal);
    mButtonBox->setStandardButtons(QDialogButtonBox::Cancel|QDialogButtonBox::Ok);
    
    mDialogGridLayout->addWidget(mButtonBox, 2, 0, 1, 1);
    
    QObject::connect(mButtonBox, SIGNAL(accepted()), this, SLOT(accept()));
    QObject::connect(mButtonBox, SIGNAL(rejected()), this, SLOT(reject()));
    
    mTabWidget->setCurrentIndex(0);
    
    //
    // Create the graphics scene to visualize
    // the colors in a 'real' scene. The hack
    // below createGraphicsScene() is necessary
    // since there is no way to actually get the
    // real background color from the surounding
    // window with Qt (backgroundRole, paletter, etc).
    // This is handled by the platform specific
    // window manager and nobody really knows what
    // that guy does.
    //
    createGraphicsScene();
    QPixmap a = grab();
    QImage img = a.toImage();
    QRgb b = img.pixel(35,35); // close to graphic view
    mGraphicsScene->setBackgroundBrush(QBrush(QColor(b)));
}

msPreferencesDialog::~msPreferencesDialog() { /* no op */}

void msPreferencesDialog::accept()
{
    //
    //  User pressed OK in the dialog. Now upload all preferences
    //  and make them permanent
    //
    
    //
    //  Game options
    //
    msSettings::mEasyStart = mEasyStartupCheckBox->isChecked();
    msSettings::mAllowQuestionableFlag = mQuestionableFlagCheckBox->isChecked();
    msSettings::mNumberOfAllowedOopsies = mAllowOopsiesCheckBox->isChecked() ? 3 : 0;
    
    //
    //  Colors
    //
    msSettings::mBackgroundColor = mSelectedBackgroundColor;
    msSettings::mWaterColor = mSelectedWaterColor;
    
    //
    //  Cell size
    //
    msSettings::mCellSize = mSelectedCellSize;
    
    //
    //  Pictograms
    //
    
    for (unsigned int i=0; i<mMineItems.size(); i++) {
        if (mMineItems[i]->isSelected()) {
            msSettings::mCurrentMineType = static_cast<msSettings::msMineType>(i);
            break;
        }
    }
    for (unsigned int i=0; i<mFlagItems.size(); i++) {
        if (mFlagItems[i]->isSelected()) {
            msSettings::mCurrentFlagType = static_cast<msSettings::msFlagType>(i);
            break;
        }
    }
    
    mBackgroundColorComboBox->setCurrentIndex(0); // 0 = current
    mWaterColorComboBox->setCurrentIndex(0); // 0 = current
    
    QDialog::accept();
}

void msPreferencesDialog::addListItem(QListWidget* list, vector<QListWidgetItem*> &vec,  bool sel, const char* str, const char* txt)
{
    QListWidgetItem *item = new QListWidgetItem(list);
    
    //
    //  Defining the icon directly via the SVG file
    //  produces terrible resolution on retina displays.
    //  This is not handled by Qt (5.3) yet.
    //  Hence we transform the SVG first into an pixmap,
    //  which gives high-res icons.
    //
    
    //
    //  Create image and fill
    //
    const int imageSize = 400; // width and height
    QImage image(imageSize, imageSize, QImage::Format_ARGB32_Premultiplied);
    image.fill(0);
    
    //
    //  Create SVG renderer and get svg size
    //
    QSvgRenderer renderer(QString(str), 0);
    QSize size = renderer.defaultSize();
    double ratio = static_cast<double>(size.width())/size.height();
    
    //
    //  Create painter
    //
    QPainter painter(&image);
    painter.setRenderHints(QPainter::Antialiasing|QPainter::SmoothPixmapTransform);
    
    
    //
    //  Fit image centered in box, keeping aspect ratio
    //
    double bw = imageSize;
    double bh = imageSize;
    double bx = 0;
    double by = 0;
    if (ratio < 1) {    // h > w
        bw = imageSize*ratio;
        bx = (imageSize-bw)/2;
    }
    else if (ratio > 1) {  // w > h
        bh = imageSize/ratio;
        by = (imageSize-bh)/2;
    }
    QRectF box(bx, by, bw, bh);
    renderer.render(&painter, box);
    
    //
    //  Set icon
    //
    QPixmap pm = QPixmap::fromImage(image);
    item->setIcon(pm);
    //item->setIcon(QIcon(str));  // use instead of above once Qt fixes QIcon(svg_file)
    
    //
    //  Set text for item
    //
    item->setText(txt);
    item->setFlags(Qt::ItemIsSelectable | Qt::ItemIsEnabled);
    item->setSelected(sel);
    item->setTextAlignment(Qt::AlignBottom|Qt::AlignHCenter);
    
    vec.push_back(item);
}

void msPreferencesDialog::backgroundColorComboActivated(int index)
{
    switch(index) {
        case(0): // current
            break;
        case(1): // default
            mSelectedBackgroundColor = msSettings::mBackgroundColorDefault;
            break;
        case(2): // new
            QColor c = QColorDialog::getColor(mSelectedBackgroundColor, this);
            if (c.isValid()) mSelectedBackgroundColor = c;
            break;
    }
    mSceneBackground->setBrush(QBrush(mSelectedBackgroundColor));
}

void msPreferencesDialog::cellSizeBoxActivated(int size)
{
    mSelectedCellSize = size;
    createGraphicsScene();
}

void msPreferencesDialog::createGraphicsScene()
{
    const int numXCells = 4;
    const int numYCells = 4;
    //
    //  Clear old view
    //  1. Remove all items from graphics scene
    //  2. Delete all cells
    //  3. Clear array holding cells
    //
    QList<QGraphicsItem*> listOfOld = mGraphicsScene->items();
    for (int ii=0; ii<listOfOld.count(); ii++) {
        mGraphicsScene->removeItem(listOfOld[ii]);
    }
    
    for (unsigned int k=0; k < mCellArray.size(); k++) {
        for (unsigned int j=0; j < mCellArray[k].size(); j++) {
            delete mCellArray[k][j];
        }
        mCellArray[k].clear();
    }
    mCellArray.clear();
    mGraphicsScene->update();
    
    //
    //  Resize cell array to correct size
    //
    mCellArray.resize(numXCells);
    for (unsigned int ix=0; ix < numYCells; ix++) {
        mCellArray[ix].resize(numYCells);
    }
    
    //
    //  Create background limited only to cell array
    //
    mSceneBackground = new QGraphicsRectItem(QRect(0, 0, numYCells*mSelectedCellSize, numYCells*mSelectedCellSize));
    QBrush bgBrush(mSelectedBackgroundColor);
    mSceneBackground->setBrush(bgBrush);
    mGraphicsScene->addItem(mSceneBackground);
    
    //
    //  Create cells and assign xy position
    //
    for (unsigned int iy=0; iy < numYCells; iy++) {
        for (unsigned int ix=0; ix < numXCells; ix++) {
            mCellArray[ix][iy] = new msCell;
            mCellArray[ix][iy]->setXY(ix, iy);
        }
    }

    QPen pen(Qt::gray);
    QBrush brush(msSettings::mWaterColor);
    for (unsigned int iy=0; iy < numYCells; iy++) {
        for (unsigned int ix=0; ix < numXCells; ix++) {
            mCellArray[ix][iy]->setXY(ix, iy);
            
            QGraphicsRectItem *item = new QGraphicsRectItem(QRect(ix*mSelectedCellSize, iy*mSelectedCellSize, mSelectedCellSize, mSelectedCellSize));
            item->setPen(pen);
            mCellArray[ix][iy]->setFrameRect(item);
            mGraphicsScene->addItem(item);
            
            QGraphicsRectItem *item2 = new QGraphicsRectItem(QRect(ix*mSelectedCellSize, iy*mSelectedCellSize, mSelectedCellSize, mSelectedCellSize));
            item2->setBrush(brush);
            item2->setPen(pen);  // same as for frame
            mCellArray[ix][iy]->setWaterRect(item2);
            mGraphicsScene->addItem(item2);
        }
    }
    mCellArray[0][0]->setIsRevealed(true);
    mCellArray[3][0]->setIsRevealed(true);
    mCellArray[3][1]->setIsRevealed(true);
    mCellArray[1][2]->setIsRevealed(true);
    mCellArray[2][2]->setIsRevealed(true);
    mCellArray[0][3]->setIsRevealed(true);
    mCellArray[2][3]->setIsRevealed(true);
    mCellArray[3][3]->setIsRevealed(true);
    
    
    //
    //  Prepare renderer for SVG numbers
    //
    for (auto iter = mNumberSvgRenderer.begin(); iter != mNumberSvgRenderer.end(); ++iter)
        delete iter->second;
    const int maxNeighbors = 8;
    for (int i=1; i<=maxNeighbors; i++)
        mNumberSvgRenderer[i] = new QSvgRenderer(msSettings::numberFileName(i));

    setGraphicsSceneText(0, 0, 3);
    setGraphicsSceneText(3, 0, 1);
    setGraphicsSceneText(3, 1, 1);
    setGraphicsSceneText(1, 2, 5);
    setGraphicsSceneText(2, 2, 4);
    setGraphicsSceneText(0, 3, 2);
    setGraphicsSceneText(2, 3, 1);
}

void msPreferencesDialog::reject()
{
    //
    //  Get here when user pressed cancel. Now we have to reset
    //  all preferences to the actual values before he or she started
    //  the dialog.
    //
    
    //
    //  Game options
    //
    mEasyStartupCheckBox->setChecked(msSettings::mEasyStart);
    mQuestionableFlagCheckBox->setChecked(msSettings::mAllowQuestionableFlag);
    mAllowOopsiesCheckBox->setChecked(msSettings::mNumberOfAllowedOopsies);
    
    //
    //  Colors
    //
    mSelectedBackgroundColor = msSettings::mBackgroundColor;
    mSelectedWaterColor = msSettings::mWaterColor;
    waterColorComboActivated(1);
    backgroundColorComboActivated(1);
    mBackgroundColorComboBox->setCurrentIndex(0); // 0 = current
    mWaterColorComboBox->setCurrentIndex(0); // 0 = current
    
    //
    //  Cell size
    //
    mSelectedCellSize = msSettings::mCellSize;
    mCellSizeSpinBox->setValue(mSelectedCellSize);
    createGraphicsScene();
    
    //
    //  Pictograms
    //
    for (unsigned int i=0; i<mMineItems.size(); i++) {
        mMineItems[i]->setSelected(false);
    }
    mMineItems[static_cast<int>(msSettings::mCurrentMineType)]->setSelected(true);
    
    for (unsigned int i=0; i<mFlagItems.size(); i++) {
        mFlagItems[i]->setSelected(false);
    }
    mFlagItems[static_cast<int>(msSettings::mCurrentFlagType)]->setSelected(true);
    
    QDialog::reject();
}

void msPreferencesDialog::setMaxCellSize(unsigned int size)
{
    unsigned int newSize = min(msSettings::mMaxCellSize, size);
    mCellSizeSpinBox->setRange(msSettings::mMinCellSize,newSize);
}

void msPreferencesDialog::setGraphicsSceneText(unsigned int ix, unsigned int iy, int num)
{
    if (!num) return;
    QGraphicsSvgItem *item = new QGraphicsSvgItem;
    item->setSharedRenderer(mNumberSvgRenderer[num]);
    item->setFlags(QGraphicsItem::ItemClipsToShape);
    item->setCacheMode(QGraphicsItem::NoCache);
    mCellArray[ix][iy]->setNumberText(item);
    mGraphicsScene->addItem(item);
    
    item->setScale(1);
    QRectF rect = item->boundingRect();
    double scale = 1;
    if (mSelectedCellSize/rect.height()<scale) scale = mSelectedCellSize/rect.height();
    if (mSelectedCellSize/rect.width()<scale) scale = mSelectedCellSize/rect.width();
    scale *= 0.8*0.8;
    item->setScale(scale);
    double dx = (mSelectedCellSize-scale*rect.width())/2.;
    double dy = (mSelectedCellSize-scale*rect.height())/2.;
    item->setPos(QPointF((ix*mSelectedCellSize)+dx, (iy*mSelectedCellSize)+dy));
}

void msPreferencesDialog::waterColorComboActivated(int index)
{
    switch(index) {
        case(0): // current
            break;
        case(1): // default
            mSelectedWaterColor = msSettings::mWaterColorDefault;
            break;
        case(2): // new
            QColor c = QColorDialog::getColor(mSelectedWaterColor, this);
            if (c.isValid()) mSelectedWaterColor = c;
            break;
    }
    
    QBrush brush(mSelectedWaterColor);
    
    for (unsigned int iy=0; iy < 4; iy++) {
        for (unsigned int ix=0; ix < 4; ix++) {
            mCellArray[ix][iy]->waterRectangle()->setBrush(brush);
        }
    }
}

